package kr.co.vividnext.sodalive.content.order

import kr.co.vividnext.sodalive.can.payment.CanPaymentService
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.content.AudioContent
import kr.co.vividnext.sodalive.content.AudioContentRepository
import kr.co.vividnext.sodalive.content.comment.AudioContentCommentRepository
import kr.co.vividnext.sodalive.content.like.AudioContentLikeRepository
import kr.co.vividnext.sodalive.content.main.GetAudioContentMainItem
import kr.co.vividnext.sodalive.member.Member
import org.springframework.beans.factory.annotation.Value
import org.springframework.stereotype.Service
import org.springframework.transaction.annotation.Transactional
import java.time.LocalDateTime

@Service
@Transactional(readOnly = true)
class OrderService(
    private val repository: OrderRepository,
    private val canPaymentService: CanPaymentService,
    private val audioContentRepository: AudioContentRepository,
    private val audioContentCommentQueryRepository: AudioContentCommentRepository,
    private val audioContentLikeQueryRepository: AudioContentLikeRepository,
    private val limitedEditionOrderRepository: LimitedEditionOrderRepository,

    @Value("\${cloud.aws.cloud-front.host}")
    private val audioContentCoverImageHost: String
) {
    @Transactional
    fun order(contentId: Long, orderType: OrderType, container: String, member: Member) {
        val content = audioContentRepository.findByIdAndActive(contentId)
            ?: throw SodaException("잘못된 콘텐츠 입니다\n다시 시도해 주세요.")
        validateOrder(memberId = member.id!!, content = content)

        val order = if (content.limited != null && content.remaining != null) {
            if (content.remaining!! <= 0) throw SodaException("해당 콘텐츠가 매진되었습니다.")
            orderLimitedEditionContent(content, member)
        } else {
            orderContent(orderType, content, member)
        }

        canPaymentService.spendCan(
            memberId = member.id!!,
            needCan = order.can,
            canUsage = CanUsage.ORDER_CONTENT,
            order = order,
            container = container
        )
    }

    private fun orderContent(orderType: OrderType, content: AudioContent, member: Member): Order {
        val order = if (content.isOnlyRental) {
            Order(type = OrderType.RENTAL)
        } else {
            Order(type = orderType)
        }
        order.member = member
        order.creator = content.member
        order.audioContent = content
        return repository.save(order)
    }

    private fun orderLimitedEditionContent(content: AudioContent, member: Member): Order {
        val order = orderContent(OrderType.KEEP, content, member)

        val sequence = limitedEditionOrderRepository.getNextSequence(content.id!!)
        val limitedEditionOrder = LimitedEditionOrder(sequence = sequence)
        limitedEditionOrder.order = order
        limitedEditionOrder.audioContent = content
        limitedEditionOrderRepository.save(limitedEditionOrder)

        content.remaining = content.remaining!! - 1

        return order
    }

    private fun validateOrder(memberId: Long, content: AudioContent) {
        if (memberId == content.member!!.id!!) throw SodaException("자신이 올린 콘텐츠는 구매할 수 없습니다.")
        if (repository.isExistOrdered(memberId = memberId, contentId = content.id!!)) {
            throw SodaException("이미 구매한 콘텐츠 입니다.")
        }
    }

    fun getAudioContentOrderList(
        member: Member,
        offset: Long,
        limit: Long
    ): GetAudioContentOrderListResponse {
        val totalCount = repository.totalAudioContentOrderListCount(
            memberId = member.id!!,
            dateTime = LocalDateTime.now()
        )
        val orderItems = repository.getAudioContentOrderList(
            dateTime = LocalDateTime.now(),
            coverImageHost = audioContentCoverImageHost,
            memberId = member.id!!,
            offset = offset,
            limit = limit
        )
            .asSequence()
            .map {
                val commentCount = audioContentCommentQueryRepository
                    .totalCountCommentByContentId(it.contentId)

                val likeCount = audioContentLikeQueryRepository
                    .totalCountAudioContentLike(it.contentId)

                it.commentCount = commentCount
                it.likeCount = likeCount
                it
            }
            .toList()

        return GetAudioContentOrderListResponse(
            totalCount = totalCount,
            items = orderItems
        )
    }

    @Transactional(readOnly = true)
    fun getAudioContentMainOrderList(memberId: Long, limit: Int): List<GetAudioContentMainItem> {
        return repository.getAudioContentMainOrderList(
            dateTime = LocalDateTime.now(),
            coverImageHost = audioContentCoverImageHost,
            memberId = memberId,
            offset = 0,
            limit = 20
        )
    }
}
