package kr.co.vividnext.sodalive.creator.admin.calculate

import com.querydsl.core.types.dsl.DateTimePath
import com.querydsl.core.types.dsl.Expressions
import com.querydsl.core.types.dsl.StringTemplate
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.admin.calculate.GetCalculateCommunityPostQueryData
import kr.co.vividnext.sodalive.admin.calculate.GetCalculateContentDonationQueryData
import kr.co.vividnext.sodalive.admin.calculate.GetCalculateContentQueryData
import kr.co.vividnext.sodalive.admin.calculate.GetCalculateLiveQueryData
import kr.co.vividnext.sodalive.admin.calculate.GetCumulativeSalesByContentQueryData
import kr.co.vividnext.sodalive.admin.calculate.QGetCalculateCommunityPostQueryData
import kr.co.vividnext.sodalive.admin.calculate.QGetCalculateContentDonationQueryData
import kr.co.vividnext.sodalive.admin.calculate.QGetCalculateContentQueryData
import kr.co.vividnext.sodalive.admin.calculate.QGetCalculateLiveQueryData
import kr.co.vividnext.sodalive.admin.calculate.QGetCumulativeSalesByContentQueryData
import kr.co.vividnext.sodalive.admin.calculate.ratio.QCreatorSettlementRatio.creatorSettlementRatio
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.can.use.QUseCan.useCan
import kr.co.vividnext.sodalive.content.QAudioContent.audioContent
import kr.co.vividnext.sodalive.content.order.QOrder.order
import kr.co.vividnext.sodalive.explorer.profile.creatorCommunity.QCreatorCommunity.creatorCommunity
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.stereotype.Repository
import java.time.LocalDateTime

@Repository
class CreatorAdminCalculateQueryRepository(private val queryFactory: JPAQueryFactory) {
    fun getCalculateLive(
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        memberId: Long
    ): List<GetCalculateLiveQueryData> {
        val formattedDate = getFormattedDate(liveRoom.beginDateTime)

        return queryFactory
            .select(
                QGetCalculateLiveQueryData(
                    member.email,
                    member.nickname,
                    formattedDate,
                    liveRoom.title,
                    liveRoom.price,
                    useCan.canUsage,
                    useCan.id.count(),
                    useCan.can.add(useCan.rewardCan).sum(),
                    creatorSettlementRatio.liveSettlementRatio
                )
            )
            .from(useCan)
            .innerJoin(useCan.room, liveRoom)
            .innerJoin(liveRoom.member, member)
            .leftJoin(creatorSettlementRatio)
            .on(member.id.eq(creatorSettlementRatio.member.id))
            .where(
                useCan.isRefund.isFalse
                    .and(liveRoom.beginDateTime.goe(startDate))
                    .and(liveRoom.beginDateTime.loe(endDate))
                    .and(liveRoom.member.id.eq(memberId))
            )
            .groupBy(liveRoom.id, useCan.canUsage, creatorSettlementRatio.liveSettlementRatio)
            .orderBy(liveRoom.id.desc(), useCan.canUsage.desc(), formattedDate.desc())
            .fetch()
    }

    fun getCalculateContentListTotalCount(
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        memberId: Long
    ): Int {
        val orderFormattedDate = getFormattedDate(order.createdAt)
        return queryFactory
            .select(audioContent.id)
            .from(order)
            .innerJoin(order.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .where(
                order.createdAt.goe(startDate)
                    .and(order.createdAt.loe(endDate))
                    .and(order.isActive.isTrue)
                    .and(order.creator.id.eq(memberId))
            )
            .groupBy(audioContent.id, order.type, orderFormattedDate, order.can)
            .orderBy(member.id.desc(), orderFormattedDate.desc(), audioContent.id.asc())
            .fetch()
            .size
    }

    fun getCalculateContentList(
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        memberId: Long,
        offset: Long,
        limit: Long
    ): List<GetCalculateContentQueryData> {
        val orderFormattedDate = getFormattedDate(order.createdAt)
        return queryFactory
            .select(
                QGetCalculateContentQueryData(
                    member.nickname,
                    audioContent.title,
                    getFormattedDate(audioContent.createdAt),
                    orderFormattedDate,
                    order.type,
                    order.can,
                    order.id.count(),
                    order.can.sum(),
                    creatorSettlementRatio.contentSettlementRatio
                )
            )
            .from(order)
            .innerJoin(order.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .leftJoin(creatorSettlementRatio)
            .on(member.id.eq(creatorSettlementRatio.member.id))
            .where(
                order.createdAt.goe(startDate)
                    .and(order.createdAt.loe(endDate))
                    .and(order.isActive.isTrue)
                    .and(order.creator.id.eq(memberId))
            )
            .groupBy(
                audioContent.id,
                order.type,
                orderFormattedDate,
                order.can,
                creatorSettlementRatio.contentSettlementRatio
            )
            .offset(offset)
            .limit(limit)
            .orderBy(member.id.desc(), orderFormattedDate.desc(), audioContent.id.asc())
            .fetch()
    }

    private fun getFormattedDate(dateTimePath: DateTimePath<LocalDateTime>): StringTemplate {
        return Expressions.stringTemplate(
            "DATE_FORMAT({0}, {1})",
            Expressions.dateTimeTemplate(
                LocalDateTime::class.java,
                "CONVERT_TZ({0},{1},{2})",
                dateTimePath,
                "UTC",
                "Asia/Seoul"
            ),
            "%Y-%m-%d"
        )
    }

    fun getCumulativeSalesByContentTotalCount(memberId: Long): Int {
        return queryFactory
            .select(audioContent.id)
            .from(order)
            .innerJoin(order.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .where(
                audioContent.member.id.eq(memberId)
                    .and(order.isActive.isTrue)
            )
            .groupBy(member.id, audioContent.id, order.can)
            .fetch()
            .size
    }

    fun getCumulativeSalesByContent(
        memberId: Long,
        offset: Long,
        limit: Long
    ): List<GetCumulativeSalesByContentQueryData> {
        return queryFactory
            .select(
                QGetCumulativeSalesByContentQueryData(
                    member.nickname,
                    audioContent.title,
                    getFormattedDate(audioContent.createdAt),
                    order.type,
                    order.can,
                    order.id.count(),
                    order.can.sum()
                )
            )
            .from(order)
            .innerJoin(order.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .where(
                audioContent.member.id.eq(memberId)
                    .and(order.isActive.isTrue)
            )
            .groupBy(member.id, audioContent.id, order.type, order.can)
            .offset(offset)
            .limit(limit)
            .orderBy(member.id.desc(), audioContent.id.desc())
            .fetch()
    }

    fun getCalculateContentDonationListTotalCount(
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        memberId: Long
    ): Int {
        val donationFormattedDate = getFormattedDate(useCan.createdAt)
        return queryFactory
            .select(audioContent.id)
            .from(useCan)
            .innerJoin(useCan.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .where(
                useCan.isRefund.isFalse
                    .and(useCan.canUsage.eq(CanUsage.DONATION))
                    .and(useCan.createdAt.goe(startDate))
                    .and(useCan.createdAt.loe(endDate))
                    .and(audioContent.member.id.eq(memberId))
            )
            .groupBy(donationFormattedDate, audioContent.id)
            .fetch()
            .size
    }

    fun getCalculateContentDonationList(
        startDate: LocalDateTime,
        endDate: LocalDateTime,
        memberId: Long,
        offset: Long,
        limit: Long
    ): List<GetCalculateContentDonationQueryData> {
        val donationFormattedDate = getFormattedDate(useCan.createdAt)
        return queryFactory
            .select(
                QGetCalculateContentDonationQueryData(
                    member.nickname,
                    audioContent.title,
                    audioContent.price,
                    getFormattedDate(audioContent.createdAt),
                    donationFormattedDate,
                    useCan.id.count(),
                    useCan.can.add(useCan.rewardCan).sum()
                )
            )
            .from(useCan)
            .innerJoin(useCan.audioContent, audioContent)
            .innerJoin(audioContent.member, member)
            .where(
                useCan.isRefund.isFalse
                    .and(useCan.canUsage.eq(CanUsage.DONATION))
                    .and(useCan.createdAt.goe(startDate))
                    .and(useCan.createdAt.loe(endDate))
                    .and(audioContent.member.id.eq(memberId))
            )
            .groupBy(donationFormattedDate, audioContent.id)
            .offset(offset)
            .limit(limit)
            .orderBy(member.id.asc(), donationFormattedDate.desc())
            .fetch()
    }

    fun getCalculateCommunityPostTotalCount(startDate: LocalDateTime?, endDate: LocalDateTime?, memberId: Long): Int {
        val formattedDate = getFormattedDate(useCan.createdAt)
        return queryFactory
            .select(creatorCommunity.id)
            .from(useCan)
            .innerJoin(useCan.communityPost, creatorCommunity)
            .innerJoin(creatorCommunity.member, member)
            .where(
                useCan.isRefund.isFalse
                    .and(useCan.canUsage.eq(CanUsage.PAID_COMMUNITY_POST))
                    .and(useCan.createdAt.goe(startDate))
                    .and(useCan.createdAt.loe(endDate))
                    .and(creatorCommunity.member.id.eq(memberId))
            )
            .groupBy(formattedDate, creatorCommunity.id)
            .fetch()
            .size
    }

    fun getCalculateCommunityPostList(
        startDate: LocalDateTime?,
        endDate: LocalDateTime?,
        memberId: Long,
        offset: Long,
        limit: Long
    ): List<GetCalculateCommunityPostQueryData> {
        val formattedDate = getFormattedDate(useCan.createdAt)
        return queryFactory
            .select(
                QGetCalculateCommunityPostQueryData(
                    member.nickname,
                    Expressions.stringTemplate("substring({0}, 1, 10)", creatorCommunity.content),
                    formattedDate,
                    creatorCommunity.price,
                    useCan.id.count(),
                    useCan.can.add(useCan.rewardCan).sum(),
                    creatorSettlementRatio.communitySettlementRatio
                )
            )
            .from(useCan)
            .innerJoin(useCan.communityPost, creatorCommunity)
            .innerJoin(creatorCommunity.member, member)
            .leftJoin(creatorSettlementRatio)
            .on(member.id.eq(creatorSettlementRatio.member.id))
            .where(
                useCan.isRefund.isFalse
                    .and(useCan.canUsage.eq(CanUsage.PAID_COMMUNITY_POST))
                    .and(useCan.createdAt.goe(startDate))
                    .and(useCan.createdAt.loe(endDate))
                    .and(creatorCommunity.member.id.eq(memberId))
            )
            .groupBy(formattedDate, creatorCommunity.id, creatorSettlementRatio.communitySettlementRatio)
            .orderBy(member.id.asc(), formattedDate.desc())
            .offset(offset)
            .limit(limit)
            .fetch()
    }
}
