package kr.co.vividnext.sodalive.creator.admin.content.series

import kr.co.vividnext.sodalive.common.ApiResponse
import kr.co.vividnext.sodalive.common.SodaException
import kr.co.vividnext.sodalive.creator.admin.content.series.content.AddingContentToTheSeriesRequest
import kr.co.vividnext.sodalive.creator.admin.content.series.content.RemoveContentToTheSeriesRequest
import kr.co.vividnext.sodalive.member.Member
import org.springframework.data.domain.Pageable
import org.springframework.lang.Nullable
import org.springframework.security.access.prepost.PreAuthorize
import org.springframework.security.core.annotation.AuthenticationPrincipal
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.PathVariable
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.PutMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RequestParam
import org.springframework.web.bind.annotation.RequestPart
import org.springframework.web.bind.annotation.RestController
import org.springframework.web.multipart.MultipartFile

@RestController
@PreAuthorize("hasRole('CREATOR')")
@RequestMapping("/creator-admin/audio-content/series")
class CreatorAdminContentSeriesController(private val service: CreatorAdminContentSeriesService) {
    @PostMapping
    fun createSeries(
        @RequestPart("image") image: MultipartFile?,
        @RequestPart("request") requestString: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.createSeries(image, requestString, member), "시리즈가 생성되었습니다.")
    }

    @PutMapping
    fun modifySeries(
        @Nullable
        @RequestPart("image")
        image: MultipartFile?,
        @RequestPart("request") requestString: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.modifySeries(image, requestString, member), "시리즈가 수정되었습니다.")
    }

    @GetMapping
    fun getSeriesList(
        pageable: Pageable,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getSeriesList(
                offset = pageable.offset,
                limit = pageable.pageSize.toLong(),
                creatorId = member.id!!
            )
        )
    }

    @GetMapping("/{id}")
    fun getDetail(
        @PathVariable id: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(service.getDetail(id = id, memberId = member.id!!))
    }

    @GetMapping("/{id}/content")
    fun getSeriesContent(
        @PathVariable id: Long,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?,
        pageable: Pageable
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.getSeriesContent(
                seriesId = id,
                offset = pageable.offset,
                limit = pageable.pageSize.toLong(),
                creatorId = member.id!!
            )
        )
    }

    @PostMapping("/add/content")
    fun addingContentToTheSeries(
        @RequestBody request: AddingContentToTheSeriesRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.addingContentToTheSeries(request, memberId = member.id!!),
            "콘텐츠가 추가되었습니다."
        )
    }

    @PutMapping("/remove/content")
    fun removeContentInTheSeries(
        @RequestBody request: RemoveContentToTheSeriesRequest,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.removeContentInTheSeries(request, memberId = member.id!!),
            "콘텐츠를 삭제하였습니다."
        )
    }

    @GetMapping("/content/search")
    fun searchContentNotInSeries(
        @RequestParam(value = "series_id") seriesId: Long,
        @RequestParam(value = "search_word") searchWord: String,
        @AuthenticationPrincipal(expression = "#this == 'anonymousUser' ? null : member") member: Member?
    ) = run {
        if (member == null) throw SodaException("로그인 정보를 확인해주세요.")

        ApiResponse.ok(
            service.searchContentNotInSeries(
                seriesId = seriesId,
                searchWord = searchWord,
                memberId = member.id!!
            )
        )
    }
}
