package kr.co.vividnext.sodalive.live.room

import com.querydsl.core.types.Projections
import com.querydsl.core.types.dsl.CaseBuilder
import com.querydsl.jpa.impl.JPAQueryFactory
import kr.co.vividnext.sodalive.can.use.CanUsage
import kr.co.vividnext.sodalive.can.use.QUseCan.useCan
import kr.co.vividnext.sodalive.can.use.QUseCanCalculate.useCanCalculate
import kr.co.vividnext.sodalive.live.room.QLiveRoom.liveRoom
import kr.co.vividnext.sodalive.live.room.QQuarterLiveRankings.quarterLiveRankings
import kr.co.vividnext.sodalive.live.room.donation.GetLiveRoomDonationItem
import kr.co.vividnext.sodalive.live.room.donation.QGetLiveRoomDonationItem
import kr.co.vividnext.sodalive.member.QMember.member
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.stereotype.Repository
import java.time.LocalDateTime
import java.time.ZoneId

@Repository
interface LiveRoomRepository : JpaRepository<LiveRoom, Long>, LiveRoomQueryRepository

interface LiveRoomQueryRepository {
    fun getLiveRoomListNow(
        offset: Long,
        limit: Long,
        timezone: String,
        memberId: Long,
        isCreator: Boolean,
        isAdult: Boolean
    ): List<LiveRoom>

    fun getLiveRoomListReservationWithDate(
        date: LocalDateTime,
        offset: Long,
        limit: Long,
        memberId: Long,
        isCreator: Boolean,
        isAdult: Boolean
    ): List<LiveRoom>

    fun getLiveRoomListReservationWithoutDate(
        timezone: String,
        memberId: Long,
        isCreator: Boolean,
        isAdult: Boolean
    ): List<LiveRoom>

    fun getLiveRoom(id: Long): LiveRoom?
    fun getLiveRoomAndAccountId(roomId: Long, memberId: Long): LiveRoom?
    fun getRecentRoomInfo(memberId: Long, cloudFrontHost: String): GetRecentRoomInfoResponse?
    fun getDonationTotal(roomId: Long): Int?
    fun getDonationList(roomId: Long, cloudFrontHost: String): List<GetLiveRoomDonationItem>
    fun getRoomActiveAndChannelNameIsNotNull(memberId: Long): List<LiveRoom>
}

class LiveRoomQueryRepositoryImpl(private val queryFactory: JPAQueryFactory) : LiveRoomQueryRepository {
    override fun getLiveRoomListNow(
        offset: Long,
        limit: Long,
        timezone: String,
        memberId: Long,
        isCreator: Boolean,
        isAdult: Boolean
    ): List<LiveRoom> {
        var where = liveRoom.channelName.isNotNull
            .and(liveRoom.channelName.isNotEmpty)
            .and(liveRoom.isActive.isTrue)
            .and(liveRoom.member.isNotNull)

        if (!isAdult) {
            where = where.and(liveRoom.isAdult.isFalse)
        }

        if (isCreator) {
            where = where.and(
                liveRoom.isAvailableJoinCreator.isTrue
                    .or(liveRoom.member.id.eq(memberId))
            )
        }

        return queryFactory
            .selectFrom(liveRoom)
            .innerJoin(liveRoom.member, member)
            .leftJoin(quarterLiveRankings).on(liveRoom.id.eq(quarterLiveRankings.roomId))
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(
                quarterLiveRankings.totalCan.coalesce(0).desc(),
                quarterLiveRankings.totalParticipants.coalesce(0).desc(),
                liveRoom.beginDateTime.desc(),
                liveRoom.member.createdAt.desc()
            )
            .fetch()
    }

    override fun getLiveRoomListReservationWithDate(
        date: LocalDateTime,
        offset: Long,
        limit: Long,
        memberId: Long,
        isCreator: Boolean,
        isAdult: Boolean
    ): List<LiveRoom> {
        var where = liveRoom.beginDateTime.goe(date)
            .and(liveRoom.beginDateTime.lt(date.plusDays(1)))
            .and(
                liveRoom.channelName.isNull
                    .or(liveRoom.channelName.isEmpty)
            )
            .and(liveRoom.isActive.isTrue)
            .and(liveRoom.member.isNotNull)

        if (!isAdult) {
            where = where.and(liveRoom.isAdult.isFalse)
        }

        if (isCreator) {
            where = where.and(
                liveRoom.isAvailableJoinCreator.isTrue
                    .or(liveRoom.member.id.eq(memberId))
            )
        }

        return queryFactory
            .selectFrom(liveRoom)
            .innerJoin(liveRoom.member, member)
            .where(where)
            .offset(offset)
            .limit(limit)
            .orderBy(liveRoom.beginDateTime.asc())
            .fetch()
    }

    override fun getLiveRoomListReservationWithoutDate(
        timezone: String,
        memberId: Long,
        isCreator: Boolean,
        isAdult: Boolean
    ): List<LiveRoom> {
        var where = liveRoom.beginDateTime.gt(
            LocalDateTime.now()
                .atZone(ZoneId.of(timezone))
                .withZoneSameInstant(ZoneId.of("UTC"))
                .toLocalDateTime()
        )
            .and(
                liveRoom.channelName.isNull
                    .or(liveRoom.channelName.isEmpty)
            )
            .and(liveRoom.isActive.isTrue)
            .and(liveRoom.member.isNotNull)

        if (!isAdult) {
            where = where.and(liveRoom.isAdult.isFalse)
        }

        if (isCreator) {
            where = where.and(
                liveRoom.isAvailableJoinCreator.isTrue
                    .or(liveRoom.member.id.eq(memberId))
            )
        }

        return queryFactory
            .selectFrom(liveRoom)
            .innerJoin(liveRoom.member, member)
            .limit(10)
            .where(where)
            .orderBy(
                CaseBuilder()
                    .`when`(member.id.eq(memberId)).then(1)
                    .otherwise(2)
                    .asc(),
                liveRoom.beginDateTime.asc()
            )
            .fetch()
    }

    override fun getLiveRoom(id: Long): LiveRoom? {
        return queryFactory
            .selectFrom(liveRoom)
            .innerJoin(liveRoom.member, member)
            .where(
                liveRoom.id.eq(id)
                    .and(liveRoom.isActive.isTrue)
            )
            .fetchFirst()
    }

    override fun getLiveRoomAndAccountId(roomId: Long, memberId: Long): LiveRoom? {
        return queryFactory
            .selectFrom(liveRoom)
            .innerJoin(liveRoom.member, member)
            .where(
                liveRoom.id.eq(roomId)
                    .and(liveRoom.isActive.isTrue)
                    .and(member.id.eq(memberId))
            )
            .fetchFirst()
    }

    override fun getRecentRoomInfo(memberId: Long, cloudFrontHost: String): GetRecentRoomInfoResponse? {
        return queryFactory
            .select(
                Projections.constructor(
                    GetRecentRoomInfoResponse::class.java,
                    liveRoom.title,
                    liveRoom.notice,
                    liveRoom.coverImage.prepend("/").prepend(cloudFrontHost),
                    liveRoom.coverImage,
                    liveRoom.numberOfPeople
                )
            )
            .from(liveRoom)
            .where(liveRoom.member.id.eq(memberId))
            .orderBy(liveRoom.id.desc())
            .limit(1)
            .fetchFirst()
    }

    override fun getDonationTotal(roomId: Long): Int? {
        return queryFactory
            .select(useCanCalculate.can.sum())
            .from(useCanCalculate)
            .innerJoin(useCanCalculate.useCan, useCan)
            .innerJoin(useCan.room, liveRoom)
            .where(
                liveRoom.id.eq(roomId)
                    .and(useCan.canUsage.eq(CanUsage.DONATION).or(useCan.canUsage.eq(CanUsage.SPIN_ROULETTE)))
                    .and(useCan.isRefund.isFalse)
            )
            .fetchOne()
    }

    override fun getDonationList(roomId: Long, cloudFrontHost: String): List<GetLiveRoomDonationItem> {
        return queryFactory
            .select(
                QGetLiveRoomDonationItem(
                    member.profileImage
                        .coalesce("profile/default-profile.png")
                        .prepend("/")
                        .prepend(cloudFrontHost),
                    member.nickname,
                    member.id.coalesce(0),
                    useCan.can.sum().add(useCan.rewardCan.sum())
                )
            )
            .from(useCan)
            .join(useCan.member, member)
            .groupBy(useCan.member)
            .where(
                useCan.room.id.eq(roomId)
                    .and(useCan.canUsage.eq(CanUsage.DONATION).or(useCan.canUsage.eq(CanUsage.SPIN_ROULETTE)))
                    .and(useCan.isRefund.isFalse)
            )
            .orderBy(useCan.can.sum().add(useCan.rewardCan.sum()).desc())
            .fetch()
    }

    override fun getRoomActiveAndChannelNameIsNotNull(memberId: Long): List<LiveRoom> {
        return queryFactory
            .selectFrom(liveRoom)
            .where(
                liveRoom.isActive.isTrue
                    .and(liveRoom.channelName.isNotNull)
                    .and(liveRoom.member.id.eq(memberId))
            )
            .fetch()
    }
}
